use utf8;
package TestDB;

# Created by DBIx::Class::Schema::Loader
# DO NOT MODIFY THE FIRST PART OF THIS FILE

use strict;
use warnings;

use base 'DBIx::Class::Schema';

__PACKAGE__->load_namespaces;


# Created by DBIx::Class::Schema::Loader v0.07037 @ 2014-05-16 00:56:29
# DO NOT MODIFY THIS OR ANYTHING ABOVE! md5sum:mo0jLHFloCZWeyQq2AAocw


# You can replace this text with custom code or comments, and it will be preserved on regeneration
1;
